from django.db import models


# Create your models here.
class ShortHand(models.Model):
    label = models.CharField(max_length=32, unique=True)
    url = models.URLField()

    def __str__(self):
        return f'{self.label} - {self.url}'



from django.db.utils import IntegrityError
from django.test import TestCase

from backend.models import ShortHand


class TestShortHand(TestCase):
    def test_creating_short_hand(self):
        s = ShortHand.objects.create(
            label='foo',
            url='https://foo.com'
        )
        self.assertIsNotNone(s)

    def test_unique_constraint(self):
        ShortHand.objects.create(
            label='foo',
            url='https://foo.com'
        )
        with self.assertRaises(IntegrityError):
            ShortHand.objects.create(
                label='foo',
                url='https://bar.com'
            )

    def test_str_representation(self):
        s = ShortHand.objects.create(
            label='foo',
            url='https://foo.com'
        )
        self.assertEqual('foo - https://foo.com', str(s))


from django.shortcuts import render, get_object_or_404, redirect
from django.http.response import HttpResponsePermanentRedirect

from backend.models import ShortHand


def index(request):
    if request.method == 'POST' and (request.POST.get('shorthand') and request.POST.get('url')):
        ShortHand.objects.create(
            label=request.POST.get('shorthand'),
            url=request.POST.get('url')
        )
        return redirect('index')

    return render(request, 'index.html', context={'shorthands': ShortHand.objects.all()})


def reroute(request, shorthand, parameter=None):
    obj = get_object_or_404(ShortHand, label=shorthand)
    url = obj.url
    print(parameter, url)
    if not parameter and '{}' in url:
        url = url.split('{}')[0]
    elif parameter:
        if '{}' in url:
            url = url.format(parameter)
        else:
            url = f'{url.rstrip("/")}/{parameter}'
    return HttpResponsePermanentRedirect(url)



from django.test import Client
from django.test import TestCase

from backend.models import ShortHand


class TestRedirect(TestCase):
    def setUp(self):
        self.client = Client()

    def test_request_shorthand_that_doesnt_exist(self):
        response = self.client.get('/foo')
        self.assertEqual(404, response.status_code)

    def test_requesting_shorthand_exists(self):
        ShortHand.objects.create(label='foo', url='https://google.com')
        response = self.client.get('/foo', follow=True)
        self.assertEqual(('https://google.com', 301), response.redirect_chain[0])

    def test_requesting_shorthand_with_wildcard_params(self):
        """
        go/foo/123 -> foo.com/search?q=123
        :return:
        """
        ShortHand.objects.create(label='foo', url='https://www.google.com/search?q={}')
        response = self.client.get('/foo/123', follow=True)
        self.assertEqual(('https://www.google.com/search?q=123', 301), response.redirect_chain[0])

    def test_requesting_shorthand_with_wildcard_params_but_no_param(self):
        ShortHand.objects.create(label='foo', url='https://www.google.com/search?q={}')
        response = self.client.get('/foo', follow=True)
        self.assertEqual(('https://www.google.com/search?q=', 301), response.redirect_chain[0])

    def test_trailing_parameter(self):
        ShortHand.objects.create(label='foo', url='https://twitch.tv')
        response = self.client.get('/foo/bar', follow=True)
        self.assertEqual(('https://twitch.tv/bar', 301), response.redirect_chain[0])

    def test_trailing_parameter_with_trailing_slash(self):
        ShortHand.objects.create(label='foo', url='https://twitch.tv/')
        response = self.client.get('/foo/bar', follow=True)
        self.assertEqual(('https://twitch.tv/bar', 301), response.redirect_chain[0])
